/*
* Copyright (c) 2010 IMS Global Learning Consortium
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
* implied. See the License for the specific language governing
* permissions and limitations under the License. 
*/

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.xml.parsers.*;
import java.awt.Cursor;
import java.util.*;
import org.w3c.dom.*;

public class ASNBrowser implements TreeSelectionListener
{
	private static boolean _debug = true;
	private static final String _searchForJurisdictionsURL = "http://asn.jesandco.org/api/1/jurisdictions";
	private Vector<String> _jurisdictionVector = new Vector<String>();
	private DefaultMutableTreeNode _root = null;
	private JFrame _frame = null;
	private boolean _commonCoreOnly = false;

	public ASNBrowser(boolean commonCoreOnly)
	{
		_frame = new JFrame("Curriculum Standards");
		_commonCoreOnly = commonCoreOnly;
		
		_frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JTree tree = new JTree(populate());
		tree.addTreeSelectionListener(this);
		JScrollPane pane = new JScrollPane(tree);
		_frame.getContentPane().add(pane);
		_frame.setSize(600, 800);
		_frame.show();
	}
	
	public static void main(String args[]) 
	{
		boolean commonCoreOnly = false;
		if (args.length > 0) {
			String arg = args[0];
			if (arg.equals("-c")) {
				commonCoreOnly = true;
			}
		}
		ASNBrowser a = new ASNBrowser(commonCoreOnly);
	}
	
	private DefaultMutableTreeNode populate()
	{
		try {
			_root = new DefaultMutableTreeNode( (_commonCoreOnly) ? "Common Core" : "Jurisdictions");
			
			DocumentBuilderFactory dbf = null;
			DocumentBuilder db = null;
			Document document = null;
			
			dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
			db = dbf.newDocumentBuilder();
			
			String xml = fetchURL(_searchForJurisdictionsURL);
			document = db.parse(new java.io.ByteArrayInputStream(xml.getBytes()));
			NodeList jurisdictionNodeList = document.getElementsByTagName("Jurisdiction");
			int numJurisdictions = jurisdictionNodeList.getLength();
			for (int i=0; i < numJurisdictions; i++) {
				Element jurisdictionElement = (Element)jurisdictionNodeList.item(i);
				String nextJurisdictionName = fetchElement(jurisdictionElement, "organizationName");
				String nextJurisdiction = fetchElement(jurisdictionElement, "organizationJurisdiction");
				if ( (!_commonCoreOnly) ||
				     (_commonCoreOnly && (nextJurisdictionName.equals("Common Core State Standards"))) ) {

					ASNBrowserJurisdictionObject userObject = new ASNBrowserJurisdictionObject(nextJurisdictionName,
																				nextJurisdiction);				
					DefaultMutableTreeNode nextNode = new DefaultMutableTreeNode();
					nextNode.setUserObject(userObject);
					_root.add(nextNode);					
					_jurisdictionVector.addElement(nextJurisdiction);
				}
			}
			return _root;
		} catch (Exception ex) {
			if (_debug) ex.printStackTrace();
		}
		return null;
	}	
	
	private String fetchURL(String query)
	{
		if (_debug) System.out.println("debug:query " + query);
		// assume already URL-encoded
		try {
			java.net.URL url = new java.net.URL(query);
			java.net.URLConnection connection = url.openConnection();
			java.net.HttpURLConnection http = (java.net.HttpURLConnection)connection;
			java.io.InputStreamReader in = new java.io.InputStreamReader(http.getInputStream());
			StringBuffer xml = new StringBuffer();
			try {
				int i = 0;
				while ( (i = in.read()) != -1 ) {
					xml.append(Character.toString((char)i));
				}
			}
			catch (Throwable t) {
				if (_debug) t.printStackTrace();
				http.disconnect();
				return null;
			}
			http.disconnect();
			if (_debug) System.out.println("debug:xml " + xml);
			return xml.toString();
		} catch (Throwable t) {
			//if (_debug) t.printStackTrace();
		}
		return null;
	}
	
	private String fetchElement(org.w3c.dom.Element element, String tag)
	{
		try {
			org.w3c.dom.NodeList elements = element.getElementsByTagName(tag);
			int numElements = elements.getLength();
			if (numElements > 0) {
				org.w3c.dom.Element e = (org.w3c.dom.Element)elements.item(0);
				if (e.hasChildNodes()) {
					return e.getFirstChild().getNodeValue();
				}
			}
		} catch (Throwable t) {
			if (_debug) t.printStackTrace();
		}
		return null;
	}	
	
	private String[] fetchElements(org.w3c.dom.Element element, String tag)
	{
		String results[] = new String[0];
		java.util.Vector<String>resultVector = new java.util.Vector<String>();
		
		try {
			org.w3c.dom.NodeList elements = element.getElementsByTagName(tag);
			int numElements = elements.getLength();
			for (int i=0; i < numElements; i++) {
				org.w3c.dom.Element e = (org.w3c.dom.Element)elements.item(i);
				if (e.hasChildNodes()) {
					resultVector.addElement(e.getFirstChild().getNodeValue());
				}
			}
			int numResults = resultVector.size();
			results = new String[numResults];
			for (int i=0; i < numResults; i++) results[i] = (String)resultVector.elementAt(i);
			return results;
		} catch (Throwable t) {
			if (_debug) t.printStackTrace();
		}
		return null;
	}		

	private String fetchAttribute(org.w3c.dom.Element element, String tag, String attr)
	{
		try {
			org.w3c.dom.NodeList elements = element.getElementsByTagName(tag);
			int numElements = elements.getLength();
			if (numElements > 0) {
				org.w3c.dom.Element e = (org.w3c.dom.Element)elements.item(0);
				return e.getAttribute(attr);
			}
		} catch (Throwable t) {
			if (_debug) t.printStackTrace();
		}
		return null;
	}	
	
	private String[] fetchAttributes(org.w3c.dom.Element element, String tag, String attr)
	{
		String results[] = new String[0];
		java.util.Vector<String>resultVector = new java.util.Vector<String>();
		
		try {
			org.w3c.dom.NodeList elements = element.getElementsByTagName(tag);
			int numElements = elements.getLength();
			for (int i=0; i < numElements; i++) {
				org.w3c.dom.Element e = (org.w3c.dom.Element)elements.item(i);
				resultVector.addElement(e.getAttribute(attr));
			}
			int numResults = resultVector.size();
			results = new String[numResults];
			for (int i=0; i < numResults; i++) results[i] = (String)resultVector.elementAt(i);
			return results;
		} catch (Throwable t) {
			if (_debug) t.printStackTrace();
		}
		return null;
	}		
	
	public void valueChanged (TreeSelectionEvent treeSelectionEvent) 
	{
		JTree treeSource = (JTree) treeSelectionEvent.getSource();		
		treeSource.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		DefaultMutableTreeNode node = (DefaultMutableTreeNode)treeSource.getLastSelectedPathComponent();
		Object o = node.getUserObject();
		if (o instanceof ASNBrowserJurisdictionObject) {
			ASNBrowserJurisdictionObject jurisdictionObject = (ASNBrowserJurisdictionObject)o;
			if (_debug) System.out.println("found " + jurisdictionObject);
			try {
				String title = null;
				String manifestUrl = null;
				
				String xml = fetchURL("http://asn.jesandco.org/api/1/documents?jurisdiction=" + jurisdictionObject.getId());		
				DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
				Document document = db.parse(new java.io.ByteArrayInputStream(xml.getBytes()));
				NodeList documentNodeList = document.getElementsByTagName("Document");
				int numDocuments = documentNodeList.getLength();
				for (int i=0; i < numDocuments; i++) {
					Element documentElement = (Element)documentNodeList.item(i);
					
					NodeList rdfNodeList = documentElement.getElementsByTagName("DocumentRdf");
					int numRdfs = rdfNodeList.getLength();
					for (int j=0; j < numRdfs; j++) {
						Element rdfElement = (Element)rdfNodeList.item(j);
						String type = rdfElement.getAttribute("type");
						if (type.equals("xml")) {
							if (rdfElement.hasChildNodes()) {
								manifestUrl = rdfElement.getFirstChild().getNodeValue();
								if (_debug) System.out.println("manifest " + manifestUrl);
							} 
						}
					}
					
					NodeList titleNodeList = documentElement.getElementsByTagName("DocumentTitle");
					if (titleNodeList.getLength() > 0) {
						Element titleElement = (Element)titleNodeList.item(0);
						title = titleElement.getFirstChild().getNodeValue();
						if (_debug) System.out.println("title " + title);
						ASNBrowserManifestObject userObject = new ASNBrowserManifestObject(title, manifestUrl);
						DefaultMutableTreeNode childNode = new DefaultMutableTreeNode();
						childNode.setUserObject(userObject);
						node.add(childNode);
					}
				}
			} catch (Exception ex) {
				//ex.printStackTrace();
			}
		} else if (o instanceof ASNBrowserManifestObject) {
			try {
				Vector<String> uriVector = new Vector<String>();
				Vector<DefaultMutableTreeNode> nodeVector = new Vector<DefaultMutableTreeNode>();
				
				// first pass to create nodes
				ASNBrowserManifestObject manifestObject = (ASNBrowserManifestObject)o;
				String xml = fetchURL(manifestObject.getUrl());		
				DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance();
				DocumentBuilder db = dbf.newDocumentBuilder();
				Document document = db.parse(new java.io.ByteArrayInputStream(xml.getBytes()));
				
				// special handling of document
				DefaultMutableTreeNode standardDocumentNode = new DefaultMutableTreeNode();
				NodeList documentNodeList = document.getElementsByTagName("asn:StandardDocument");
				if (documentNodeList.getLength() == 1) {
					Element documentElement = (Element)documentNodeList.item(0);
					String uri = documentElement.getAttribute("rdf:about");
					if (_debug) System.out.println("standard document uri " + uri);
					String authorityStatus = fetchAttribute(documentElement,"asn:authorityStatus","rdf:resource");
					String indexingStatus = fetchAttribute(documentElement,"asn:indexingStatus","rdf:resource");
					String educationLevels[] = fetchAttributes(documentElement,"dcterms:educationLevel","rdf:resource");
					String educationLevel = "";
					for (int j=0; j < educationLevels.length; j++) {
						int index = educationLevels[j].lastIndexOf("/");
						if (index != -1) {
							educationLevel += educationLevels[j].substring(index) + ", ";
						}
					}
					String subject = fetchAttribute(documentElement,"dcterms:subject","rdf:resource");
					String description = fetchElement(documentElement,"dcterms:description");
					String language = fetchAttribute(documentElement,"dcterms:language","rdf:resource");
					String identifier = fetchAttribute(documentElement,"asn:identifier","rdf:resource");
					
					ASNBrowserStatementObject userObject = new ASNBrowserStatementObject(uri,
																		    authorityStatus,
																		    indexingStatus,
																		    educationLevel,
																		    subject,
																		    description,
																		    language,
																		    identifier);
					standardDocumentNode.setUserObject(userObject);
					node.add(standardDocumentNode);
					uriVector.addElement(uri);
					nodeVector.addElement(standardDocumentNode);
				}
				
				documentNodeList = document.getElementsByTagName("asn:Statement");
				int numDocuments = documentNodeList.getLength();
				if (_debug) System.out.println("num documents " + numDocuments);
				for (int i=0; i < numDocuments; i++) {
					Element documentElement = (Element)documentNodeList.item(i);
					String uri = documentElement.getAttribute("rdf:about");
					String authorityStatus = fetchAttribute(documentElement,"asn:authorityStatus","rdf:resource");
					String indexingStatus = fetchAttribute(documentElement,"asn:indexingStatus","rdf:resource");
					String educationLevels[] = fetchAttributes(documentElement,"dcterms:educationLevel","rdf:resource");
					String educationLevel = "";
					for (int j=0; j < educationLevels.length; j++) {
						int index = educationLevels[j].lastIndexOf("/");
						if (index != -1) {
							educationLevel += educationLevels[j].substring(index) + ", ";
						}
					}
					String subject = fetchAttribute(documentElement,"dcterms:subject","rdf:resource");
					String description = fetchElement(documentElement,"dcterms:description");
					if (_debug) System.out.println("description " + uri + " " + description);
					String language = fetchAttribute(documentElement,"dcterms:language","rdf:resource");
					String identifier = fetchAttribute(documentElement,"asn:identifier","rdf:resource");
					
					ASNBrowserStatementObject userObject = new ASNBrowserStatementObject(uri,
																		    authorityStatus,
																		    indexingStatus,
																		    educationLevel,
																		    subject,
																		    description,
																		    language,
																		    identifier);
					DefaultMutableTreeNode childNode = new DefaultMutableTreeNode();
					childNode.setUserObject(userObject);
					uriVector.addElement(uri);
					nodeVector.addElement(childNode);
				}

				// special handling of document
				documentNodeList = document.getElementsByTagName("asn:StandardDocument");
				if (documentNodeList.getLength() == 1) {
					Element documentElement = (Element)documentNodeList.item(0);
					String childUri[] = fetchAttributes(documentElement, "gemq:hasChild", "rdf:resource");
					if (childUri != null) {
						for (int j=0; j < childUri.length; j++) {
							String nextUri = childUri[j];
							int index = uriVector.indexOf(nextUri);
							if (index != -1) {
								DefaultMutableTreeNode childNode = (DefaultMutableTreeNode)nodeVector.elementAt(index);
								standardDocumentNode.add(childNode);
							}
						}
					}
				}

				// second pass to create child relationships
				documentNodeList = document.getElementsByTagName("asn:Statement");
				numDocuments = documentNodeList.getLength();
				for (int i=0; i < numDocuments; i++) {
					Element documentElement = (Element)documentNodeList.item(i);
					// parent node is current node
					String uri = documentElement.getAttribute("rdf:about");
					if (uri != null) {
						int parentIndex = uriVector.indexOf(uri);
						if (parentIndex != -1) {
							DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode)nodeVector.elementAt(parentIndex);

							// hook up any children
							String childUri[] = fetchAttributes(documentElement, "gemq:hasChild", "rdf:resource");
							if (childUri != null) {
								for (int j=0; j < childUri.length; j++) {
									String nextUri = childUri[j];
									if (_debug) System.out.println("next uri " + nextUri);
									int childIndex = uriVector.indexOf(nextUri);
									if (childIndex != -1) {
										DefaultMutableTreeNode childNode = (DefaultMutableTreeNode)nodeVector.elementAt(childIndex);
										parentNode.add(childNode);
									}
								}
							}
						}
					}					
				}				
			} catch (Exception ex) {
				ex.printStackTrace();
				System.exit(1);
			}
		} else if (node.isLeaf() && (o instanceof ASNBrowserStatementObject)) {
			// pop-up something
			JOptionPane.showInputDialog(_frame,
								   "Associated Curriculum Standard",
								   "ASN URI",
								   JOptionPane.PLAIN_MESSAGE,
								   null,
								   null,
								   ((ASNBrowserStatementObject)o).getUri());
		}
		treeSource.setCursor(Cursor.getDefaultCursor());
	}
}
